@@ -1,15 +1,24 @@
+import inspect
+from matplotlib import docstring
+
 # Log styles
 from . import styles_lhcb as style
 from . import label as label_base
 from .label import lumitext
-import mplhep._deprecate as deprecate
-from matplotlib import docstring
+import mplhep
 
 __all__ = [style, lumitext]
 
 
 # Experiment wrappers:
 def _lhcb_text(text="", **kwargs):
+    for key, value in dict(mplhep.rcParams.text._get_kwargs()).items():
+        if (
+            value is not None
+            and key not in kwargs.keys()
+            and key in inspect.getfullargspec(label_base._exp_text).args
+        ):
+            kwargs[key] = value
     return label_base._exp_text(
         "LHCb",
         text=text,
@@ -22,6 +31,13 @@ def _lhcb_text(text="", **kwargs):
 
 
 def _lhcb_label(**kwargs):
+    for key, value in dict(mplhep.rcParams.label._get_kwargs()).items():
+        if (
+            value is not None
+            and key not in kwargs.keys()
+            and key in inspect.getfullargspec(label_base._exp_label).args
+        ):
+            kwargs[key] = value
     return label_base._exp_label(
         exp="LHCb",
         italic=(False, False),
@@ -32,17 +48,6 @@ def _lhcb_label(**kwargs):
     )
 
 
-# Change to snake_case
-@deprecate.deprecate("Naming convention is changing. Use ``mplhep.lhcb.label``.")
-def lhcblabel(*args, **kwargs):
-    return _lhcb_label(*args, **kwargs)
-
-
-@deprecate.deprecate("Naming convention is changing. Use ``mplhep.lhcb.text``.")
-def lhcbtext(**kwargs):
-    return _lhcb_text(**kwargs)
-
-
 @docstring.copy(label_base._exp_text)
 def text(*args, **kwargs):
     return _lhcb_text(*args, **kwargs)
